//
//
//
/******************************************************/
/*         DigestPlayer                               */
/*                                                    */
/******************************************************/

import Information.*;
import Interface.*;
import Analyzer.*;
import GUI.*;

import java.io.*;
import java.util.*;
import java.awt.*;

public class DigestPlayer extends ApplicationLogInterface
                               implements GameControlPanelEventListener {

    /******************************************/
    /*         instance variable              */
    /******************************************/

    PlayModeAnalyzer pma;
    KickAnalyzerDontUseFlag ka;
    PassAnalyzer pa;
    InterceptAnalyzer ia;
    DribbleAnalyzer da;
    ShootAnalyzer sa;
    GoalAnalyzer ga;

    JikkyoInterface ji;

    FieldCanvas fc;
    GameControlPanel gcp;
    Frame f;

    /******************************************/
    /*            constructor                 */
    /******************************************/

    public DigestPlayer(String s){

	super(s);

	pma = new PlayModeAnalyzer(bsi);
	ka  = new KickAnalyzerDontUseFlag(bsi);
	pa = new PassAnalyzer(ka,pma);
	ia = new InterceptAnalyzer(ka,pma);
	da = new DribbleAnalyzer(ka,pma);
	sa = new ShootAnalyzer(bsi, ka);
	ga = new GoalAnalyzer(bsi, pma, ka);

	ji = new JikkyoInterface(pa,da,ia,sa,ga);

    }

    /******************************************/
    /*             method                     */
    /******************************************/

    public void init(){
	
	super.init();
	
	f = new Frame();
	f.setLayout(new BorderLayout());

	FieldCanvas c = new FieldCanvas(bsi);
	GameStatusPanel gsp = new GameStatusPanel(bsi);
	gsp.init();

	gcp = new GameControlPanel(this);
	gcp.init();

	f.add("Center",c);
	f.add("North",gsp);
	f.add("South",gcp);

	f.setVisible(true);

    }

    public void run(){

	while(true){

	    try{

		//logfile
		lpi.parse();

		bsi.setChanged();
		bsi.notifyObservers();

		sleep(100);

	    }catch(IOException e){

		e.printStackTrace();

	    }catch(InterruptedException e){

		e.printStackTrace();
		
	    }
		
	}

    }

    /*********************************************/
    /* GameControlPanelEventListener       */
    /*********************************************/

    public void stopPushed(){};
    public void closePushed(){};
    public void pausePushed(){};
    public void restartPushed(){};

    /*********************************************/
    /*                main method                */ 
    /*********************************************/

    public static void main(String argv[]){

        DigestPlayer dp = new DigestPlayer(argv[0]);

	dp.init();
	dp.callDLL();

	try{
	  Thread.sleep(10000);
	}catch(Exception e){
	  e.printStackTrace();
	}

	dp.start();

    }

    /*********************************************/
    /* for TalkMachine                           */
    /*********************************************/

    public native int TalkMachineMain();

    void callDLL(){

        System.loadLibrary("TalkMachineDLL");

    }

    public class JikkyoInterface implements Observer{

	int dribble_hassei_count = 0;
	int init = 0;

	public JikkyoInterface(PassAnalyzer pa, DribbleAnalyzer da, InterceptAnalyzer ia,
				 ShootAnalyzer sa, GoalAnalyzer ga){

	    pa.addObserver(this);
	    da.addObserver(this);
	    ia.addObserver(this);
	    sa.addObserver(this);
	    ga.addObserver(this);

	}
    
	public void update(java.util.Observable o, Object arg){

	    if (o instanceof PassAnalyzer){

		Pass pass = (Pass)arg;
		int side = pass.getSide();
		int pUnum = pass.getPasserUnum();

		sayMessage(pUnum+"p ppass");

	    }else if (o instanceof DribbleAnalyzer){
	   
		double SHIKIICHI = 5.0;

		Integer itg = (Integer)arg;
		int i = itg.intValue();

		if (i == DribbleAnalyzer.NEW_DRIBBLE){

		    dribble_hassei_count = 0;

		}

		DribbleAnalyzer da = (DribbleAnalyzer)o;
	    
		if (da.getCurrentDribbleDistance() >= (SHIKIICHI*i)){

		    dribble_hassei_count++;

		    int side = da.getCurrentDribble().getSide();
		    int dUnum = da.getCurrentDribble().getMember() + side*11;
		}

	    }else if (o instanceof InterceptAnalyzer){

		Intercept inter = (Intercept)arg;

		int side = inter.getIntercepterSide();
		int iUnum = inter.getIntercepterUnum() + side*11;
	    
		sayMessage(iUnum+"p pintercept");

	    }else if (o instanceof ShootAnalyzer){

		Shoot shoot = (Shoot)arg;
		int side = shoot.getSide();
		int sUnum = shoot.getMember() + side*11;

		sayMessage(sUnum+"p pshoot");

	    }else if (o instanceof GoalAnalyzer){

		Goal goal = (Goal)arg;
		int side = goal.getSide();
		int gUnum = goal.getMember() + side*11;

		sayMessage(gUnum+"p pgoal");

	    }

	}

	private void sayMessage(String str){

	    //for debug
	    System.out.println(str);

	    //TalkMachineMain(init, str);

	    if (init == 0){
		init = 1;
	    }
	    
	}

    }
}









